package cn.ninegame.genericframework.basic;

import android.app.Application;
import android.content.Context;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Looper;
import android.util.Log;

import java.io.File;
import java.util.HashMap;
import java.util.List;

import cn.ninegame.genericframework.config.ModulePathManager;
import cn.ninegame.genericframework.module.IModuleManifest;
import cn.ninegame.genericframework.module.ModuleCenter;
import cn.ninegame.genericframework.module.ModulePreloader;
import cn.ninegame.genericframework.module.flashbuild.FlashbuildReceiver;
import cn.ninegame.genericframework.tools.ApkUtils;
import cn.ninegame.genericframework.tools.dynamicload.NativeLibManager;
import cn.ninegame.genericframework.ui.FragmentCenter;

/**
 * Created by lingcong on 2015/3/18.
 * Changed by zhonghui on 2017/04/01
 * 职责：承担了IOC中容器的角色，负责初始化整个框架层，并给相关的核心类设置依赖关系，注入依赖对象。
 */
public final class FrameworkFacade {
    private static String TAG = FrameworkFacade.class.getSimpleName();
    private  static FrameworkFacade sInstance = null;
    private FrameworkLifecycleHandler frameworkLifecycleHandler;
    //建立消息名和与MockMessagehandler的映射关系
    private static HashMap<String,IMockMessageHandler> mMockMessageHandlerMap = new HashMap<String, IMockMessageHandler>(2);

    private boolean mIsStarted = false;
    private boolean mIsPreloaded = false;
    private FrameworkConfig mFrameworkConfig;

    ControllerCenter controllerCenter;
    ModuleCenter moduleCenter;
    FlashbuildReceiver flashbuildReceiver = null;


    private  FrameworkFacade() {
    }

    public static FrameworkFacade getInstance() {
        if (sInstance == null) {
            synchronized (FrameworkFacade.class){
                if (sInstance == null) {
                    sInstance = new FrameworkFacade();
                }
            }
        }

        return  sInstance;
    }

    public synchronized void registerMockHandler(String messageName,IMockMessageHandler handler){
        mMockMessageHandlerMap.put(messageName,handler);
    }

    public synchronized boolean isStarted(){
        return mIsStarted;
    }

    public synchronized void setStarted(boolean status){
        this.mIsStarted = status;
    }

    public static void sendMessage(int messageId){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return;
        }
        FrameworkFacade.getInstance().getEnvironment().sendMessage(messageId);
    }
    public static void sendMessage(int messageId, Bundle messageData){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return;
        }
        FrameworkFacade.getInstance().getEnvironment().sendMessage(messageId,messageData);
    }

    public static void sendMessageForResult(int messageId, Bundle messageData,IResultListener listener){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return;
        }
        FrameworkFacade.getInstance().getEnvironment().sendMessageForResult(messageId,messageData,listener);
    }

    public static Bundle sendMessageSync(int messageId){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return null;
        }
        return FrameworkFacade.getInstance().getEnvironment().sendMessageSync(messageId);
    }

    public static Bundle sendMessageSync(int messageId, Bundle messageData){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return null;
        }
        return FrameworkFacade.getInstance().getEnvironment().sendMessageSync(messageId,messageData);
    }


    public static void sendMessage(String messageName){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return;
        }
        if(FrameworkService.getInstance().mIsDebug){
            if(mMockMessageHandlerMap.containsKey(messageName)){
                mMockMessageHandlerMap.get(messageName).handleMessage(messageName,null,null);
                Log.e(TAG,"Frameowrk message " + messageName+ "already mocked");
                return;
            }
        }
        FrameworkFacade.getInstance().getEnvironment().sendMessage(messageName);
    }
    public static void sendMessage(String messageName, Bundle messageData){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return;
        }
        if(FrameworkService.getInstance().mIsDebug){
            if(mMockMessageHandlerMap.containsKey(messageName)){
                mMockMessageHandlerMap.get(messageName).handleMessage(messageName,messageData,null);
                Log.e(TAG,"Frameowrk message " + messageName+ "already mocked");
                return;
            }
        }
        FrameworkFacade.getInstance().getEnvironment().sendMessage(messageName,messageData);
    }

    public static void sendMessage(String messageName, Bundle messageData, IMessageContext messageContext){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return;
        }
        if(FrameworkService.getInstance().mIsDebug){
            if(mMockMessageHandlerMap.containsKey(messageName)){
                mMockMessageHandlerMap.get(messageName).handleMessage(messageName,messageData,null);
                Log.e(TAG,"Frameowrk message " + messageName+ "already mocked");
                return;
            }
        }
        FrameworkFacade.getInstance().getEnvironment().sendMessage(messageName,messageContext,messageData);
    }

    public static void sendMessageForResult(String messageName, Bundle messageData,IResultListener listener){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return;
        }
        if(FrameworkService.getInstance().mIsDebug){
            if(mMockMessageHandlerMap.containsKey(messageName)){
                mMockMessageHandlerMap.get(messageName).handleMessage(messageName,messageData,listener);
                Log.e(TAG,"Frameowrk message " + messageName+ "already mocked");
                return;
            }
        }
        FrameworkFacade.getInstance().getEnvironment().sendMessageForResult(messageName,messageData,listener);
    }

    public static void sendMessageForResult(String messageName, Bundle messageData,IResultListener listener,IMessageContext messageContext){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return;
        }
        if(FrameworkService.getInstance().mIsDebug){
            if(mMockMessageHandlerMap.containsKey(messageName)){
                mMockMessageHandlerMap.get(messageName).handleMessage(messageName,messageData,listener);
                Log.e(TAG,"Frameowrk message " + messageName+ "already mocked");
                return;
            }
        }
        FrameworkFacade.getInstance().getEnvironment().sendMessageForResult(messageName,messageContext,messageData,listener);
    }

    public static Bundle sendMessageSync(String messageName){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return null;
        }
        if(FrameworkService.getInstance().mIsDebug){
            if(mMockMessageHandlerMap.containsKey(messageName)){
                Log.e(TAG,"Frameowrk message " + messageName+ "already mocked");
                return mMockMessageHandlerMap.get(messageName).handleMessageSync(messageName,null);
            }
        }
        return FrameworkFacade.getInstance().getEnvironment().sendMessageSync(messageName);
    }

    public static Bundle sendMessageSync(String messageName, Bundle messageData){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return null;
        }
        if(FrameworkService.getInstance().mIsDebug){
            if(mMockMessageHandlerMap.containsKey(messageName)){
                Log.e(TAG,"Frameowrk message " + messageName+ "already mocked");
                return mMockMessageHandlerMap.get(messageName).handleMessageSync(messageName,messageData);
            }
        }
        return FrameworkFacade.getInstance().getEnvironment().sendMessageSync(messageName,messageData);
    }

    public static Bundle sendMessageSync(String messageName, Bundle messageData,IMessageContext messageContext){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return null;
        }
        if(FrameworkService.getInstance().mIsDebug){
            if(mMockMessageHandlerMap.containsKey(messageName)){
                Log.e(TAG,"Frameowrk message " + messageName+ "already mocked");
                return mMockMessageHandlerMap.get(messageName).handleMessageSync(messageName,messageData);
            }
        }
        return FrameworkFacade.getInstance().getEnvironment().sendMessageSync(messageName,messageContext,messageData);
    }

    public static void sendNotification(Notification notification){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return;
        }
        FrameworkFacade.getInstance().getEnvironment().sendNotification(notification);
    }

    public static void sendNotification(int notificationId,Bundle bundleData){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return;
        }
        FrameworkFacade.getInstance().getEnvironment().sendNotification(notificationId,bundleData);
    }

    public static void sendNotification(String notificationName,Bundle bundleData){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return;
        }
        FrameworkFacade.getInstance().getEnvironment().sendNotification(notificationName,bundleData);
    }

    public static int getMessageIdByName(String messageName){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return -1;
        }
        return FrameworkFacade.getInstance().getEnvironment().getMessageIdByName(messageName);
    }

    public static Context getContext(){
        if(!FrameworkFacade.getInstance().isStarted()){
            Log.e(TAG,"Frameowrk is not ready");
            return null;
        }
        return FrameworkFacade.getInstance().getEnvironment().getApplicationContext();
    }

    /**
     * 启动框架，运行作为Launcher的Controller,默认限制在主线程启动,
     * @param applicationContext        应用程序的ApplicationContext
     * @param apkFile                   framework所在apk的当前路径
     * @param manifests
     * @param optionalFrameworkConfig   可选配置
     */
    public synchronized void start(Context applicationContext, File apkFile, IModuleManifest[] manifests, FrameworkConfig optionalFrameworkConfig) {
        start(applicationContext,apkFile,manifests,optionalFrameworkConfig,true);
    }
    /**
     * 启动框架，运行作为Launcher的Controller
     * @param applicationContext        应用程序的ApplicationContext
     * @param apkFile                   framework所在apk的当前路径
     * @param manifests
     * @param optionalFrameworkConfig   可选配置
     * @param checkMainThread 是否限制初始化在主线程
     */
    public synchronized void start(Context applicationContext, File apkFile, IModuleManifest[] manifests, FrameworkConfig optionalFrameworkConfig,boolean checkMainThread) {
        if (isStarted()) {
            return;
        }

        if (manifests == null || manifests.length == 0) {
            return;
        }

        if (applicationContext == null) {
            return;
        }
        if(checkMainThread){
            checkThread();
        }
        initFromConfig(optionalFrameworkConfig);

        frameworkLifecycleHandler = new FrameworkLifecycleHandler();
        FrameworkService.frameworkListeners.add(frameworkLifecycleHandler);

        ModulePathManager.getInstance().init(applicationContext, apkFile);

        moduleCenter = new ModuleCenter();
        moduleCenter.setLoadModuleListener(optionalFrameworkConfig.getLoadModuleListener());

        controllerCenter = new ControllerCenter();
        EnvironmentImpl environment = new EnvironmentImpl();
        MsgBroker msgBroker = new MsgBroker(checkMainThread);

        environment.setMsgBroker(msgBroker);
        environment.setApplicationContext(applicationContext);
        EnvironmentInterceptor envInterceptor = null;
        if (optionalFrameworkConfig.getEnvironmentInterceptor() == null) {
            envInterceptor = new DefaultEnvironmentInteceptor();
        }else {
            envInterceptor = optionalFrameworkConfig.getEnvironmentInterceptor();
        }
        envInterceptor.setEnvironment(environment);
        environment.setInterceptor(envInterceptor);

        msgBroker.setControllerCenter(controllerCenter);

        controllerCenter.setEnvironment(environment);
        controllerCenter.setModuleCenter(moduleCenter);
        controllerCenter.setMsgBroker(msgBroker);

        FragmentCenter fragmentCenter = new FragmentCenter();
        fragmentCenter.setEnv(environment);
        fragmentCenter.setModuleCenter(moduleCenter);
        environment.setFragmentCenter(fragmentCenter);

        NotificationCenter notificationCenter = new NotificationCenter();
        notificationCenter.init(manifests);
        environment.setNotificationCenter(notificationCenter);

        moduleCenter.setControllerCenter(controllerCenter);
        moduleCenter.setFragmentCenter(fragmentCenter);
        boolean bInit = moduleCenter.init(manifests);
        RuntimeVariables.androidApplication = (Application) applicationContext.getApplicationContext();
        RuntimeVariables.publicKeys  = ApkUtils.getApkPublicKey(apkFile.getPath());
        RuntimeVariables.sRawClassLoader = applicationContext.getClassLoader();
        RuntimeVariables.mContext = applicationContext;
        //判断高达处在SDk模式下,暂时屏蔽 1.1.0,将会在1.1.1版本时候打开
        if(!RuntimeVariables.androidApplication.getApplicationInfo().publicSourceDir.equals(RuntimeVariables.mContext.getApplicationInfo().publicSourceDir)){
            NativeLibManager.init(RuntimeVariables.mContext);
        }
        this.setStarted(true);
        if (bInit) {
            if (optionalFrameworkConfig.isDebug()) {
                registerFlashbuildReceiver(applicationContext);
            }
            environment.sendMessageSync(FrameworkMessage.LAUNCHER_CONTROLLER_INVOKE);
        }
    }

    private void initFromConfig(FrameworkConfig frameworkConfig) {
        if (frameworkConfig == null) {
            mFrameworkConfig = new FrameworkConfig();
        } else {
            mFrameworkConfig = frameworkConfig;

            FrameworkService frameworkService = FrameworkService.getInstance();
            frameworkService.setDebug(frameworkConfig.isDebug());
            frameworkService.getService(CacheManager.class).setCacheManagerDelegate(frameworkConfig.getCacheDelegate());
            frameworkService.getService(SecurityManager.class).setSecurity(frameworkConfig.getSecurityDelegate());
            frameworkService.getService(ExceptionManager.class).addExceptionHandler(frameworkConfig.getExceptionHandler());
            frameworkService.getService(LogManager.class).setCacheManagerDelegate(frameworkConfig.getLogHandler());
        }
    }

    public FrameworkConfig getConfig() {
        return mFrameworkConfig;
    }

    public Environment getEnvironment(){
        if(controllerCenter != null){
            return controllerCenter.getEnviroment();
        }
        return  null;
    }

    public void startPreload(List<Integer> msgs,long interval, float cpuBusyThreshold) {
        if (!mIsPreloaded) {
            new ModulePreloader(controllerCenter.getEnviroment(), moduleCenter, msgs, interval, cpuBusyThreshold).start();
            mIsPreloaded = true;
        }
    }

    private static void checkThread() {
        if(Thread.currentThread().getId()!=Looper.getMainLooper().getThread().getId()){
            throw new RuntimeException("Gundam framework must be started in ui thread");
        }
    }

    /**
     * 设置统计类
     * @param stat
     */
    public void setStat(IStatDelegate stat){
        FrameworkService.getInstance().getService(StatManager.class).setStat(stat);
    }

    /**
     * 添加异常处理类
     * @param exceptionHandler
     */
    public void addExceptionHandler(IExceptionHandler exceptionHandler){
        FrameworkService.getInstance().getService(ExceptionManager.class).addExceptionHandler(exceptionHandler);
    }

    /**
     * 移除异常处理类
     * @param exceptionHandler
     */
    public void removeExceptionHandler(IExceptionHandler exceptionHandler){
        FrameworkService.getInstance().getService(ExceptionManager.class).removeExceptionHandler(exceptionHandler);
    }

    public void destroy() {
        if (moduleCenter != null) {
            moduleCenter.destroy();
        }
        unRegisterFlashbuildReceiver(getEnvironment().getApplicationContext());
    }

    private void registerFlashbuildReceiver(Context context) {
        IntentFilter filter = new IntentFilter();
        filter.addAction(FlashbuildReceiver.INTENT_ACTION);

        flashbuildReceiver = new FlashbuildReceiver();
        context.registerReceiver(flashbuildReceiver,filter);
    }

    private void unRegisterFlashbuildReceiver(Context context) {
        if (flashbuildReceiver != null) {
            context.unregisterReceiver(flashbuildReceiver);
        }
    }

    private void checkingThread(boolean strict){
        if(Thread.currentThread().getId()!=Looper.getMainLooper().getThread().getId()){
            if(strict){
                throw new RuntimeException("can not install bundle in ui thread");
            }else{
                Log.w("Gundam","can not install bundle in ui thread");
            }
        }
    }
}
