package cn.ninegame.genericframework.tools;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Build;

import java.io.File;
import java.io.IOException;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 *   升级覆盖检查
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/10/8
 */
public class AppUpgradeUtil {

    public static final String SP_NAME = "app_upgrade";
    public static final String KEY_MODIFY_TIME  = "_modify_time";
    public static final String KEY_CRC  = "_crc";

    /**
     * 根据源apk文件的最后修改时间判断是否属于覆盖安装
     * @param context
     * @param businessKey
     * @return
     *  返回 true 表示覆盖安装
     *  返回 false 表示非覆盖安装
     */
    public static boolean checkByLastModifyTime(Context context, String businessKey){
        String spKey = businessKey + KEY_MODIFY_TIME;

        File sourceApk  = new File(context.getApplicationInfo().sourceDir);

        long currentTime = getTimeStamp(sourceApk);

        SharedPreferences sp = context.getSharedPreferences(SP_NAME, Context.MODE_MULTI_PROCESS | Context.MODE_PRIVATE);
        long recordModifyTime = sp.getLong(spKey, -1L);

        if (recordModifyTime == currentTime) {
            return false;
        } else {
            SharedPreferences.Editor editor = sp.edit();
            editor.putLong(spKey, currentTime);
            SharedPreferencesCommit(editor);

            return true;
        }
    }

    /**
     * 根据源apk文件的最后修改时间和文件CRC值判断是否属于覆盖安装， 两个条件满足一个就认为是覆盖安装
     * @param context
     * @param businessKey
     * @return
     *  返回 true 表示覆盖安装
     *  返回 false 表示非覆盖安装
     */
    public static boolean checkByLastModifyTimeOrCrc(Context context, String businessKey){
        String modifyKey = businessKey + KEY_MODIFY_TIME;
        String crcKey = businessKey + KEY_CRC;

        File sourceApk  = new File(context.getApplicationInfo().sourceDir);

        long currentTime = getTimeStamp(sourceApk);
        long currentCrc = getZipCrc(sourceApk);

        SharedPreferences sp = context.getSharedPreferences(SP_NAME, Context.MODE_MULTI_PROCESS | Context.MODE_PRIVATE);
        long recordModifyTime = sp.getLong(modifyKey, -1L);
        long recordCrc = sp.getLong(crcKey, -1L);

        if (recordModifyTime != currentTime || recordCrc != currentCrc) {
            SharedPreferences.Editor editor = sp.edit();
            editor.putLong(modifyKey, currentTime);
            editor.putLong(crcKey, currentCrc);
            SharedPreferencesCommit(editor);
            return true;
        } else {
            return false;
        }
    }


    static long getZipCrc(File archive){
        long computedValue = -1L;
        try {
            computedValue = ZipUtil.getZipCrc(archive);
        } catch (IOException e) {
            e.printStackTrace();
        }
        if(computedValue == -1L) {
            --computedValue;
        }

        return computedValue;
    }

    static long getTimeStamp(File archive) {
        long timeStamp = archive.lastModified();
        if(timeStamp == -1L) {
            --timeStamp;
        }

        return timeStamp;
    }

    private static void  SharedPreferencesCommit(SharedPreferences.Editor editor){
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.GINGERBREAD) {
            editor.commit();
        } else {
            editor.apply();
        }
    }
}
