package cn.ninegame.genericframework.tools.dynamicload;

import android.os.Build;

import java.io.File;
import java.io.IOException;
import java.util.zip.ZipFile;

import cn.ninegame.genericframework.tools.ReflectHelper;
import dalvik.system.DexFile;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 *  通过反射构造DexPathList$Element实例
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/4/29
 */
public class ElementFactory {

    public static Object getElement(String dexPathList, File file, boolean isDirectory, File zip, DexFile dex){
        Object element = null;
        if (Build.VERSION.SDK_INT >= 18) {
            //public Element(File file, boolean isDirectory, File zip, DexFile dexFile)
            element = getElementV18(dexPathList, file, isDirectory, zip, dex);
            if (element == null) {
                element = getElementV17(dexPathList, file, zip, dex);
            }
            if (element == null) {
                element = getElementLessThan17(dexPathList, file, zip, dex);
            }
        } else if (Build.VERSION.SDK_INT >= 17) {
            //public Element(File file, File zip, DexFile dexFile)
            element = getElementV17(dexPathList, file, zip, dex);
            if (element == null) {
                element = getElementV18(dexPathList, file, isDirectory, zip, dex);
            }
            if (element == null) {
                element = getElementLessThan17(dexPathList, file, zip, dex);
            }
        } else {
            //public Element(File file, ZipFile zipFile, DexFile dexFile)
            element = getElementLessThan17(dexPathList, file, zip, dex);
            if (element == null) {
                element = getElementV18(dexPathList, file, isDirectory, zip, dex);
            }
            if (element == null) {
                element = getElementV17(dexPathList, file, zip, dex);
            }
        }

        return element;
    }

    private static Object getElementV18(String dexPathList, File file, boolean isDirectory, File zip, DexFile dex){
        return ReflectHelper.loadClass(dexPathList + "$Element", new Class[]{File.class, boolean.class, File.class, DexFile.class}, file, isDirectory, zip, dex);
    }

    private static Object getElementV17(String dexPathList, File file, File zip, DexFile dex){
        return ReflectHelper.loadClass(dexPathList + "$Element", new Class[]{File.class, File.class, DexFile.class}, file, zip, dex);
    }

    private static Object getElementLessThan17(String dexPathList, File file, File zip, DexFile dex){
        try {
            ZipFile zipFile = (zip == null ? null : new ZipFile(zip));
            return ReflectHelper.loadClass(dexPathList + "$Element", new Class[]{File.class, ZipFile.class, DexFile.class}, file, zipFile, dex);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }
}
