package cn.ninegame.genericframework.ui;

import android.app.Activity;
import android.content.Context;
import android.content.res.Configuration;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Looper;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;

import cn.ninegame.genericframework.basic.*;
import cn.ninegame.genericframework.module.IModuleComponent;
import cn.ninegame.genericframework.module.IModuleEntry;
import cn.ninegame.genericframework.module.ModuleType;
import cn.ninegame.genericframework.tools.LOG;

/**
 * ****************************************************************************
 * Copyright @ 2009 - 2015 www.9game.cn All Rights Reserved
 * <p/>
 * Creation    : 2015-03-18
 * Author      : lihq@ucweb.com
 * Description : BaseFragment兼具了View和业务Controller的角色，是PAC模式中的A, 即抽象层
 * ****************************************************************************
 */
public abstract class BaseFragment extends Fragment implements View.OnClickListener, INotify, IModuleComponent {

    private static final String TAG = "BaseFragment";
    //用于保存传过来的listener（类似intent）
    private IResultListener mIntentResultListener;

    //用于保存当前fragment start另一个fragment的listener，onResume时回调
    @Deprecated
    private IResultListener mSourceResultListener;

    private IBundleArgumentListener mBundleArgumentListener;
    private static final long ITEM_CLICK_COOLING_TIME_IN_MS = 500;
    private long mLastItemClickTimeInMS;
    private Bundle mBundle = new Bundle();

    private Environment mEnv;
    private IModuleEntry mModuleEntry;

    private ActivityContextWrapper mActivityContextWrapper;

    private boolean mUseAnim;
    private boolean mUserActivityAnim;
    private boolean mCovered;

    public int mEnterAnimRes;
    public int mExitAnimRes;
    public int mPopEnterAnimRes;
    public int mPopExitAnimRes;

    /**
     *  Fragment启动模式 -> 新建Fragment并压入栈中；
     */
    public static final int LAUNCHER_MODE_NORMAL = 0;
    /**
     *  Fragment启动模式 -> 当前Activity的栈顶Fragment栈中匹配到Fragment，则调用栈顶Fragment的onNewIntent方法
     */
    public static final int LAUNCHER_MODE_SINGLE_TOP = 1;
    /**
     *  Fragment启动模式 -> 当前Activity的Fragment栈中匹配到Fragment，则将目标Fragment之上的其他Fragment弹出，没有匹配到则新建Fragment并压入栈中
     */
    public static final int LAUNCHER_MODE_CLEAR_TOP = 2;
    /**
     * Fragment启动模式 -> 弹出当前Fragment，并新建Fragment压入栈中
     */
    public static final int LAUNCHER_MODE_START_WITH_POP = 3;
    /**
     * Fragment启动模式 -> 隐藏当前Fragment，并新建Fragment压入栈中
     */
    public static final int LAUNCHER_MODE_START_WITH_HIDE = 4;

    /**
     * Fragment被系统回收调用 onSaveInstanceState时，保存状态的Key
     */
    public static final String FRAGMENT_STATE = "fragment_state";

    public abstract Class getHostActivity();

    public int getContainer(){
        return android.R.id.content;
    }

    public void setUseAnim(boolean useAnim){
        mUseAnim = useAnim;
    }

    public boolean isUseAnim(){
        return mUseAnim;
    }

    public void setUserActivityAnim(Boolean userActivityAnim) {
        mUserActivityAnim = userActivityAnim;
    }

    public boolean isUserActivityAnim() {
        return mUserActivityAnim;
    }

    public void setCustomAnimations(int enterAnimRes, int exitAnimRes, int popEnterAnimRes, int popExitAnimRes){
        mEnterAnimRes = enterAnimRes;
        mExitAnimRes = exitAnimRes;
        mPopEnterAnimRes = popEnterAnimRes;
        mPopExitAnimRes = popExitAnimRes;
    }

    @Deprecated
    @Override
    public void setArguments(Bundle args) {
        super.setArguments(args);
    }

    public void setBundleArguments(Bundle bundle){
        mBundle = bundle;
    }

    public Bundle getBundleArguments(){
        return mBundle;
    }

    public void setSourceResultListener(IResultListener listener){
        mSourceResultListener = listener;
    }

    public IResultListener getSourceResultListener(){
        return mSourceResultListener;
    }

    public String getName(){
        return this.getClass().getName();
    }

    public void setResultBundle(Bundle bundle){
        if(mIntentResultListener != null){
            mIntentResultListener.setResultBundle(bundle);
        }
    }

    void setResultListener(IResultListener listener) {
        this.mIntentResultListener = listener;
    }

    public IResultListener getResultListener(){
        return mIntentResultListener;
    }

    public void setBundleArgumentListener(IBundleArgumentListener listener){
        this.mBundleArgumentListener = listener;
    }

    @Override
    public void setEnvironment(Environment environment) {
        mEnv = environment;
    }

    @Override
    public Environment getEnvironment() {
        return mEnv;
    }

    public void sendMessage(String messageId){
        if(mEnv != null){
            mEnv.sendMessage(messageId);
        }
    }

    public void sendMessage(String messageId, Bundle messageData){
        if(mEnv != null){
            mEnv.sendMessage(messageId, messageData);
        }
    }

    public void sendMessageForResult(String messageId, Bundle messageData,IResultListener listener){
        if(mEnv != null){
            mEnv.sendMessageForResult(messageId, messageData, listener);
        }
    }

    public Bundle sendMessageSync(String messageId, Bundle messageData) {
        if(mEnv != null){
            return mEnv.sendMessageSync(messageId,messageData);
        }
        return null;
    }


    public void sendNotification(String notification, Bundle extraData){
        if(mEnv != null){
            mEnv.sendNotification(notification, extraData);
        }
    }

    public void registerNotification(String notifycationId, INotify notify){
        if(mEnv != null){
            mEnv.registerNotification(notifycationId, notify);
        }
    }

    public void unregisterNotification(String notifycationId, INotify notify){
        if(mEnv != null){
            mEnv.unregisterNotification(notifycationId, notify);
        }
    }

    private void registerNotification() {
        RegisterNotification[] notifications = getNotification();

        if(mEnv != null) {
            for (int i = 0; i < notifications.length; i++) {
                mEnv.registerNotification(notifications[i].name(), this);
            }
        }
    }

    private void unregisterNotification(){
        RegisterNotification[] notifications = getNotification();

        if(mEnv != null) {
            for (int i = 0; i < notifications.length; i++) {
                mEnv.unregisterNotification(notifications[i].name(), this);
            }
        }
    }

    private RegisterNotification[] getNotification(){
        RegisterNotification[] notifications = {};
        try {
            RegisterNotifications annotation = this.getClass().getAnnotation(RegisterNotifications.class);

            if (annotation != null) {
                notifications = annotation.value();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return notifications;
    }

    /**
     * 处理返回事件,#goBack()或者按物理back键后 都会回调到 #onBackPressed()
     * @return true为拦截框架的回退处理，false为不拦截。框架的默认回退处理是#popCurrentFragment()
     * 默认是不拦截
     * */
    public boolean onBackPressed() {
        return false;
    }

    /**
     * 调用宿主Activity的{@link Activity#onBackPressed()}方法
     */
    public void onActivityBackPressed() {
        Activity activity = getActivity();
        if (activity != null) {
            hideKeyboard();
            mEnv.getCurrentActivity().onBackPressed();
        }
    }

    public void hideKeyboard(){
        Activity activity = null;
        if(mEnv != null){
            activity = mEnv.getCurrentActivity();
        }

        if(activity == null){
            activity = getActivity();
        }

        if (activity != null && activity.getCurrentFocus() != null) {
            IBinder binder = activity.getCurrentFocus().getWindowToken();
            if( binder != null){
                hideKeyboard(activity, binder);
            }
        }
    }

    private boolean hideKeyboard(Context ctx, IBinder binder) {
        InputMethodManager imm = (InputMethodManager) ctx.getSystemService(Context.INPUT_METHOD_SERVICE);

        if (imm != null && imm.isActive()) {
            return imm.hideSoftInputFromWindow(binder, 0);
        }

        return false;
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (mEnv == null) {
            FrameworkFacade.getInstance().getEnvironment().reInitFragment(this);
        }
        registerNotification();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return super.onCreateView(inflater, container, savedInstanceState);
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        view.setClickable(true);
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    private void performResultListener() {
        if(mIntentResultListener != null){
            if(mEnv.getCurrentActivity() != null) {
                Handler mainHandler = new Handler(Looper.getMainLooper());
                mainHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        if(mIntentResultListener != null) {
                            mIntentResultListener.performOnResult();
                            mIntentResultListener.setResultBundle(null);
                            mIntentResultListener = null;
                        }
                    }
                });
            }
        }
    }

    @Override
    public void onDestroy() {
        try {
            performResultListener();
        } catch (Exception e) {
            e.printStackTrace();
        }
        super.onDestroy();
        unregisterNotification();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
    }

    @Override
    public void onClick(View view) {
        //防止用户点击速度过快而引起多个窗口or对话框弹出.//FIXME lihq
        if (overCoolingTime()){
            this.mLastItemClickTimeInMS  = System.currentTimeMillis();
        }else{
            return;
        }
    }

    private boolean overCoolingTime() {
        return (System.currentTimeMillis() - this.mLastItemClickTimeInMS) >= ITEM_CLICK_COOLING_TIME_IN_MS;
    }

    @Override
    public void onNotify(Notification notification) {

    }

    /**
     * 当Fragment的参数发生改变（一般发生在顶层的fragment相同，为避免新建Fragment而重新改变Fragment参数）
     * @param bundle
     */
    void notifyArgumentChanged(Bundle bundle){
        if(mBundleArgumentListener != null){
            mBundleArgumentListener.onBundleArgumentsChanged(bundle);
        }
    }

    @Override
    public void setModuleEntry(IModuleEntry entry) {
        if (mModuleEntry != null) {
            throw new IllegalArgumentException("Module entry has already set!");
        }
        mModuleEntry = entry;
    }

    @Override
    public Context getContext() {
        if (mModuleEntry == null){
            throw new RuntimeException(String.format("getContext,  mModuleEntry is null, className = %s", getName()));
        }
        Activity activity = mEnv.getCurrentActivity();
        if (mModuleEntry.getModuleType() == ModuleType.LOCAL) {
            return activity;
        } else {
            ActivityContextWrapper wrapper = mActivityContextWrapper;
            if (wrapper == null || !wrapper.isSameActivity(activity)) {
                wrapper = new ActivityContextWrapper(activity, mModuleEntry.getContext());
                mActivityContextWrapper = wrapper;
            }
            return wrapper;
        }
    }

    @Override
    public String getModuleRootPath(){
        return mModuleEntry.getModuleRootPath();
    }

    @Override
    public String getModuleDataPath(){
        return mModuleEntry.getModuleDataPath();
    }

    @Override
    public String getModuleSoPath(){
        return mModuleEntry.getModuleSoPath();
    }

    @Override
    public LayoutInflater getLayoutInflater(Bundle savedInstanceState) {
        if (mModuleEntry == null){
            throw new RuntimeException(String.format("getLayoutInflater,  mModuleEntry is null, className = %s", getName()));
        }
        if (mModuleEntry.getModuleType() == ModuleType.LOCAL) {
            return super.getLayoutInflater(savedInstanceState);
        } else {
            return LayoutInflater.from(getContext());
        }
    }

    public BaseFragment loadFragment(String clzName){
        if (mModuleEntry == null){
            throw new RuntimeException(String.format("loadFragment,  mModuleEntry is null, className = %s", getName()));
        }
        BaseFragment fragment = mModuleEntry.loadFragment(clzName);
        fragment.setEnvironment(mEnv);
        return fragment;
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        DisplayMetrics dm = getResources().getDisplayMetrics();
        getResources().updateConfiguration(newConfig,dm);
    }

    public BaseDialogFragment loadDialogFragment(String clzName){
        if (mModuleEntry == null){
            throw new RuntimeException(String.format("loadDialogFragment,  mModuleEntry is null, className = %s", getName()));
        }
        BaseDialogFragment fragment = mModuleEntry.loadDialogFragment(clzName);
        fragment.setEnvironment(mEnv);
        return fragment;
    }

    /**
     * Fragment是否被覆盖
     * @return
     */
    public boolean isCovered() {
        return mCovered;
    }

    public void setCovered(boolean covered) {
        this.mCovered = covered;
    }

    /**
     * 当前Fragment被其他Fragment覆盖， 回调该方法
     */
    public void onCover(){
        LOG.d(TAG, String.format("onCover class=%s", getClass().getSimpleName()));
    }

    /**
     *  覆盖在当前Fragment之上的Fragment出栈完成后， 回调该方法
     */
    public void onUncover(){
        LOG.d(TAG, String.format("onUncover class=%s", getClass().getSimpleName()));
    }

    /**
     * 弹出栈顶Fragment，当后退栈为1的时候，直接结束当前Activity
     */
    public void popCurrentFragment(){
        if (getActivity() instanceof BaseActivity){
            BaseActivity currentActivity = (BaseActivity) getActivity();
            if(currentActivity.isForeground() && !currentActivity.isFinishing()){
                currentActivity.popCurrentFragment();
            }
        }
    }

    /**
     * 弹出栈顶Fragment，当Fragment后退栈为1的时候，弹出Fragment后会出现空白的Activity
     */
    public void popCurrentFragmentOnly(){
        if (getActivity() instanceof BaseActivity){
            BaseActivity currentActivity = (BaseActivity) getActivity();
            if(currentActivity.isForeground() && !currentActivity.isFinishing()){
                currentActivity.popCurrentFragmentOnly();
            }
        }
    }

    public void onNewIntent(Bundle bundle){}
}
