package cn.ninegame.genericframework.ui;

import android.app.Activity;
import android.app.AlertDialog;
import android.graphics.Color;
import android.support.annotation.NonNull;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.util.Log;
import android.util.SparseArray;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.TextView;

import java.util.List;
import java.util.Stack;

import cn.ninegame.genericframework.basic.FrameworkFacade;

/**
 * <p>Title: ucweb</p>
 * <p>
 * <p>Description: </p>
 * 调试工具
 * <p>Copyright: Copyright (c) 2018</p>
 * <p>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2018/9/6
 */
public class DebugStackTool {

    public void attachToActivity(@NonNull final Activity activity){
        View decorView = activity.getWindow().getDecorView();
        FrameLayout contentParent = (FrameLayout) decorView.findViewById(android.R.id.content);

        final TextView textView = new TextView(activity);
        textView.setText("stack");
        textView.setGravity(Gravity.CENTER);
        textView.setBackgroundColor(Color.RED);
        textView.setPadding(20, 30, 20, 30);
//
        final FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        params.topMargin = 1500;
        params.leftMargin = 20;

        textView.setLayoutParams(params);
        textView.setClickable(true);

        // 拖动事件
        textView.setOnTouchListener(new View.OnTouchListener(){
            int lastX, lastY;
            int paramX, paramY;

            public boolean onTouch(View v, MotionEvent event) {
                switch (event.getAction()) {
                    case MotionEvent.ACTION_DOWN:
                        lastX = (int) event.getRawX();
                        lastY = (int) event.getRawY();
                        paramX = params.leftMargin;
                        paramY = params.topMargin;
                        return false;
                    case MotionEvent.ACTION_MOVE:
                        int dx = (int) event.getRawX() - lastX;
                        int dy = (int) event.getRawY() - lastY;
                        params.leftMargin = paramX + dx;
                        params.topMargin = paramY + dy;
                        // 更新悬浮窗位置
                        textView.setLayoutParams(params);
                        return true;
                    case MotionEvent.ACTION_UP:
                        return false;
                }
                return true;
            }
        });

        textView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String task = dumpTaskRecord();
                TextView tv = new TextView(activity);
                tv.setText(task);
                AlertDialog dialog = new AlertDialog.Builder(activity)
                        .setView(tv)
                        .setNegativeButton("关闭", null)
                        .create();
                dialog.show();
                Log.d("BaseActivity", dumpTaskRecord());
//                Toast.makeText(textView.getContext(), "堆栈已打印", Toast.LENGTH_SHORT).show();
            }
        });

        contentParent.addView(textView);

        // 堆栈变化的时候， 将调试图标调到最前面
        final FragmentManager fragmentManager = ((FragmentActivity)activity).getSupportFragmentManager();
        fragmentManager.addOnBackStackChangedListener(new FragmentManager.OnBackStackChangedListener() {
            @Override
            public void onBackStackChanged() {
                if (fragmentManager.getBackStackEntryCount() >= 1) {
                    ViewGroup viewGroup = (ViewGroup) textView.getParent();
                    if (viewGroup != null) {
                        viewGroup.removeView(textView);
                        viewGroup.addView(textView);
                    }
                }
            }
        });
    }

    private String dumpTaskRecord() {
        boolean isInCurrentTask = false;
        //dump all the tasks
        SparseArray<Stack<Activity>> taskArray = FrameworkFacade.getInstance().getEnvironment().getActivityStack();
        Activity currentActivity = FrameworkFacade.getInstance().getEnvironment().getCurrentActivity();
        StringBuilder sb = new StringBuilder();
        if (taskArray != null && currentActivity != null) {
            int key = 0;
            Stack<Activity> stack;

            for (int i = 0; i < taskArray.size(); i++) {
                key = taskArray.keyAt(i);
                stack = taskArray.get(key);

                if (key == currentActivity.getTaskId()) {
                    sb.append("[*] Task Id:" + key).append('\n');
                    isInCurrentTask = true;
                } else {
                    sb.append("[ ] Task Id:" + key).append('\n');
                    isInCurrentTask = false;
                }

                //dump activities
                if (stack != null && stack.size() > 0) {
                    for (int location = stack.size() - 1; location >= 0; location--) {
                        Activity activity = stack.get(location);
                        String activityName = activity.getClass().getSimpleName();
                        if (isInCurrentTask && location == (stack.size() - 1)) {
                            sb.append("    [*] Activity:" + activityName).append('\n');
                        } else {
                            sb.append("    [ ] Activity:" + activityName).append('\n');
                        }

                        //dump fragments
                        if (activity instanceof FragmentActivity) {
                            FragmentActivity fActivity = (FragmentActivity) activity;
                            FragmentManager manager = fActivity.getSupportFragmentManager();
                            List<Fragment> fs = manager.getFragments();
                            if (fs != null && fs.size() > 0) {
                                for (int index = fs.size() - 1; index >= 0; index--) {
                                    Fragment f = fs.get(index);
                                    if (f != null) {
                                        if (isInCurrentTask && (location == stack.size() - 1) && (index == (fs.size() - 1))) {
                                            sb.append("        [*] Fragment:" + f.getClass().getSimpleName()).append(f.isHidden() ? "(h)" : "").append('\n');
                                            dumpSonFragments(sb, f, true, 3);
                                        } else {
                                            sb.append("        [ ] Fragment:" + f.getClass().getSimpleName()).append(f.isHidden() ? "(h)" : "").append('\n');
                                            dumpSonFragments(sb, f, false, 3);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        } else {
            sb.append("No TaskRecord.");
        }
        return sb.toString();
    }

    private String TAB_TAG = "    ";
    //递归dump Fragment下的子Fragments
    private void dumpSonFragments(StringBuilder sb, Fragment f, boolean isCurrent, int tabNum) {
        //子fragments
        FragmentManager childFragmentManager = f.getChildFragmentManager();
        if (childFragmentManager != null) {
            List<Fragment> childFs = childFragmentManager.getFragments();
            if (childFs != null && childFs.size() > 0) {

                for (int index = childFs.size() - 1; index >= 0; index--) {
                    Fragment cf = childFs.get(index);
                    if (cf != null) {
                        for (int i = 0; i < tabNum; i++) {
                            sb.append(TAB_TAG);
                        }

                        int newTabNum = tabNum + 1;
                        if (isCurrent && (index == (childFs.size() - 1))) {
                            sb.append("[*] Fragment:" + cf.getClass().getSimpleName()).append(f.isHidden() ? "(h)" : "").append('\n');
                            dumpSonFragments(sb, cf, true, newTabNum);
                        } else {
                            sb.append("[ ] Fragment:" + cf.getClass().getSimpleName()).append(f.isHidden() ? "(h)" : "").append('\n');
                            dumpSonFragments(sb, cf, false, newTabNum);
                        }
                    }
                }
            }
        }
    }
}
